@@ -35,7 +35,7 @@ struct Command {
 
 fn print_help() {
     print!("Input/output is hex unless toggled to decimal with 'x'
-h            This help
+h            This (h)elp
 <Enter>      Print current byte(s) and move forward to next set of byte(s)
 3d4          Move to byte number 3d4 and print from there
 +            Move 1 byte forward and print from there
@@ -50,27 +50,28 @@ $            Move to last byte and print it
                and print.
 /            Perform last search again starting at next byte
 ?            Perform last search (backwards) again starting at previous byte
-k            Delete (kill) byte at current index and print new line of byte(s)
-7dk          Move to byte 7d, delete that byte, and print from there.
-1d,72k       Move to byte 1d, delete bytes 1d - 72 inclusive, and print from there.
+k            Delete/(k)ill byte at current index and print new line of byte(s)
+7dk          Move to byte 7d, (k)ill that byte, and print from there.
+1d,72k       Move to byte 1d, (k)ill bytes 1d - 72 inclusive, and print from there.
 /deadbeef/k  If the bytes de ad be ef exist after the current index, move there,
-               delete those bytes, and print.
-i            Prompt you to write out bytes which will be inserted at current index
+               (k)ill those bytes, and print.
+i            Prompt you to write out bytes which will be (i)nserted at current index
 72i          Move to byte number 72 and prompt you to enter bytes which will be
-               inserted there.
+               (i)nserted there.
 /deadbeef/i  If the bytes de ad be ef exist after the current index, move there
-               and prompt you to enter bytes which will be inserted there.
-12,3dp       Print bytes 12 - 3d inclusive, move to leftmost byte printed on the
-               last line.
+               and prompt you to enter bytes which will be (i)nserted there.
+12,3dp       (p)rint bytes 12 - 3d inclusive, move to byte 12
 m            Toggle whether or not characters are printed after bytes
-n            Toggle whether or not byte numbers are printed before bytes
-o            Toggle using color
-p            Print current line of byte(s) (depending on 'W')
-s            Print state of all toggles and 'W'idth
-x            Toggle interpreting inputs and displaying output as hex or decimal
-w            Actually write changes to the file on disk
-W3d          Print a linebreak every 3d bytes [Default 0x10]
-q            quit
+n            Toggle whether or not byte (n)umbers are printed before bytes
+o            Toggle using c(o)lor
+p            (p)rint current line of byte(s) (depending on 'W')
+s            Print (s)tate of all toggles and 'W'idth
+t3d          Print 0x3d lines of con(t)extual bytes after current line [Default 0]
+T3d          Print 0x3d lines of con(T)extual bytes before current line [Default 0]
+x            Toggle interpreting inputs and displaying output as he(x) or decimal
+w            Actually (w)rite changes to the file on disk
+W3d          Set (W)idth to 0x3d.  i.e. print a linebreak every 3d bytes [Default 0x10]
+q            (q)uit
 ");
 }
 
@@ -112,6 +113,8 @@ impl Command {
         let re_offset_index = Regex::new(r"^ *(?P<sign>[-+])(?P<offset>[0-9A-Fa-f]+) *(?P<the_rest>.*) *$").unwrap();
         let re_matches_nothing = Regex::new(r"^a\bc").unwrap();
         let re_width = Regex::new(r"^ *W *(?P<width>[0-9A-Fa-f]+) *$").unwrap();
+        let re_before_context = Regex::new(r"^ *T *(?P<before_context>[0-9A-Fa-f]+) *$").unwrap();
+        let re_after_context = Regex::new(r"^ *t *(?P<after_context>[0-9A-Fa-f]+) *$").unwrap();
 
         let is_blank_line          = re_blank_line.is_match(line);
         let is_single_char_command = re_single_char_command.is_match(line);
@@ -125,6 +128,8 @@ impl Command {
         let is_specified_index     = re_specified_index.is_match(line);
         let is_offset_index        = re_offset_index.is_match(line);
         let is_width               = re_width.is_match(line);
+        let is_before_context      = re_before_context.is_match(line);
+        let is_after_context       = re_after_context.is_match(line);
 
         let re = if is_blank_line {
             re_blank_line
@@ -159,6 +164,12 @@ impl Command {
         else if is_offset_index {
             re_offset_index
         }
+        else if is_before_context {
+            re_before_context
+        }
+        else if is_after_context {
+            re_after_context
+        }
         else if is_width {
             re_width
         }
@@ -337,6 +348,36 @@ impl Command {
             })
         }
 
+        else if is_before_context {
+            let caps = caps.unwrap();
+            let given = caps.name("before_context").unwrap().as_str();
+            if let Ok(before_context) = usize::from_str_radix(given, state.radix) {
+              Ok(Command{
+                  range: (usize::from(before_context), usize::from(before_context)),
+                  command: 'T',
+                  args: vec![],
+              })
+            }
+            else {
+                Err(format!("Can't interpret {} as a number", given))
+            }
+        }
+
+        else if is_after_context {
+            let caps = caps.unwrap();
+            let given = caps.name("after_context").unwrap().as_str();
+            if let Ok(after_context) = usize::from_str_radix(given, state.radix) {
+              Ok(Command{
+                  range: (usize::from(after_context), usize::from(after_context)),
+                  command: 't',
+                  args: vec![],
+              })
+            }
+            else {
+                Err(format!("Can't interpret {} as a number", given))
+            }
+        }
+
         else if is_width {
             // println!("is_width");
             let caps = caps.unwrap();
@@ -537,7 +578,7 @@ fn minuses(state:&mut State, num_minuses:usize) -> Result<usize, String> {
     }
     else {
         state.index -= num_minuses;
-        state.print_bytes(state.range());
+        state.print_bytes_sans_context(state.range());
         Ok(state.index)
     }
 }
@@ -558,7 +599,7 @@ fn pluses(state:&mut State, num_pluses:usize) -> Result<usize, String> {
                 }
                 else {
                     state.index += num_pluses;
-                    state.print_bytes(state.range());
+                    state.print_bytes_sans_context(state.range());
                     Ok(state.index)
                 }
             },
@@ -621,6 +662,8 @@ pub fn actual_runtime(filename:&str, quiet:bool, color:bool) -> i32 {
         show_chars: true,
         unsaved_changes: false,
         index: 0,
+        before_context: 0,
+        after_context: 0,
         width: NonZeroUsize::new(16).unwrap(),
         all_bytes: all_bytes,
         // TODO calculate based on longest possible index
@@ -632,7 +675,7 @@ pub fn actual_runtime(filename:&str, quiet:bool, color:bool) -> i32 {
         println!("h for help\n");
         println!("{}", state);
         println!();
-        state.print_bytes(state.range());
+        state.print_bytes_sans_context(state.range());
     }
 
     loop {
@@ -662,7 +705,7 @@ pub fn actual_runtime(filename:&str, quiet:bool, color:bool) -> i32 {
                     'g' => {
                         match ec::move_to(&mut state, command.range.0) {
                             Ok(_) => {
-                                state.print_bytes(state.range());
+                                state.print_bytes();
                             },
                             Err(error) => {
                                 println!("? ({})", error);
@@ -715,7 +758,7 @@ pub fn actual_runtime(filename:&str, quiet:bool, color:bool) -> i32 {
                                 }
                                 state.all_bytes = new;
                                 state.unsaved_changes = true;
-                                state.print_bytes(state.range());
+                                state.print_bytes_sans_context(state.range());
                             },
                             Err(error) => {
                                 println!("? ({})", error);
@@ -740,7 +783,7 @@ pub fn actual_runtime(filename:&str, quiet:bool, color:bool) -> i32 {
                         right_half = right_half.split_off(command.range.1 - command.range.0 + 1);
                         state.all_bytes.append(&mut right_half);
                         state.index = command.range.0;
-                        state.print_bytes(state.range());
+                        state.print_bytes_sans_context(state.range());
                     },
 
                     /* Toggle showing char representations of bytes */
@@ -799,7 +842,7 @@ pub fn actual_runtime(filename:&str, quiet:bool, color:bool) -> i32 {
                                 }
                                 else {
                                     state.index = first_byte_to_show_index;
-                                    state.print_bytes((first_byte_to_show_index,
+                                    state.print_bytes_sans_context((first_byte_to_show_index,
                                             last_byte_to_show_index));
                                 }
                             },
@@ -818,12 +861,7 @@ pub fn actual_runtime(filename:&str, quiet:bool, color:bool) -> i32 {
 
                         skip_bad_range!(command, state.all_bytes);
                         state.index = command.range.0;
-                        if let Some(last_left_col_index) = state.print_bytes(state.range()) {
-                            state.index = last_left_col_index;
-                        }
-                        else {
-                            println!("? (bad range {:?}", state.range());
-                        }
+                        state.print_bytes();
                     },
 
                     /* Print byte(s) with range */
@@ -835,8 +873,8 @@ pub fn actual_runtime(filename:&str, quiet:bool, color:bool) -> i32 {
 
                         skip_bad_range!(command, state.all_bytes);
                         state.index = command.range.0;
-                        if let Some(last_left_col_index) = state.print_bytes((command.range.0, command.range.1)) {
-                        state.index = last_left_col_index;
+                        if state.print_bytes_sans_context((command.range.0, command.range.1)).is_some() {
+                            state.index = command.range.0;
                         }
                         else {
                             println!("? (bad range {:?}", command.range);
@@ -850,7 +888,7 @@ pub fn actual_runtime(filename:&str, quiet:bool, color:bool) -> i32 {
                             continue;
                         };
 
-                        state.print_bytes(state.range());
+                        state.print_bytes();
                     },
 
                     /* Quit */
@@ -863,6 +901,16 @@ pub fn actual_runtime(filename:&str, quiet:bool, color:bool) -> i32 {
                         println!("{}", state);
                     },
 
+                    /* Change after_context */
+                    't' => {
+                        state.after_context = usize::from(command.range.0);
+                    },
+
+                    /* Change before_context */
+                    'T' => {
+                        state.before_context = usize::from(command.range.0);
+                    },
+
                     /* Write out */
                     'w' => {
                         let result = std::fs::write(filename, &state.all_bytes);
